@@ -1,4 +1,4 @@
-/* json-gobject.h - JSON GObject integration
+/* json-gobject.c - JSON GObject integration
  * 
  * This file is part of JSON-GLib
  * Copyright (C) 2007  OpenedHand Ltd.
@@ -40,8 +40,8 @@
 #include <stdlib.h>
 
 #include "json-types-private.h"
+#include "json-gobject-private.h"
 
-#include "json-gobject.h"
 #include "json-parser.h"
 #include "json-generator.h"
 
@@ -173,7 +173,218 @@ flags_from_string (GType        type,
   return ret;
 }
 
-static gboolean
+static GObject *
+json_gobject_new (GType       gtype,
+                  JsonObject *object)
+{
+  JsonSerializableIface *iface = NULL;
+  JsonSerializable *serializable = NULL;
+  gboolean deserialize_property;
+  GList *members, *members_left, *l;
+  guint n_members;
+  GObjectClass *klass;
+  GObject *retval;
+  GArray *construct_params;
+  gint i;
+
+  klass = g_type_class_ref (gtype);
+
+  n_members = json_object_get_size (object);
+  members = json_object_get_members (object);
+  members_left = NULL;
+
+  /* first pass: construct and construct-only properties; here
+   * we cannot use Serializable because we don't have an
+   * instance yet; we use the default implementation of
+   * json_deserialize_pspec() to deserialize known types
+   *
+   * FIXME - find a way to allow deserialization for these
+   * properties
+   */
+  construct_params = g_array_sized_new (FALSE, FALSE, sizeof (GParameter), n_members);
+
+  for (l = members; l != NULL; l = l->next)
+    {
+      const gchar *member_name = l->data;
+      GParamSpec *pspec;
+      GParameter param = { NULL, };
+      JsonNode *val;
+      gboolean res = FALSE;
+
+      pspec = g_object_class_find_property (klass, member_name);
+      if (!pspec)
+        goto next_member;
+
+      if (!(pspec->flags & G_PARAM_CONSTRUCT_ONLY) ||
+          !(pspec->flags & G_PARAM_CONSTRUCT_ONLY))
+        goto next_member;
+
+      if (!(pspec->flags & G_PARAM_WRITABLE))
+        goto next_member;
+
+      g_value_init (&param.value, G_PARAM_SPEC_VALUE_TYPE (pspec));
+
+      val = json_object_get_member (object, member_name);
+      res = json_deserialize_pspec (&param.value, pspec, val);
+      if (!res)
+        g_value_unset (&param.value);
+      else
+        {
+          param.name = g_strdup (pspec->name);
+
+          g_array_append_val (construct_params, param);
+
+          continue;
+        }
+
+    next_member:
+      members_left = g_list_prepend (members_left, l->data);
+    }
+
+  retval = g_object_newv (gtype,
+                          construct_params->len,
+                          (GParameter *) construct_params->data);
+
+  /* free the contents of the GArray */
+  for (i = 0; i < construct_params->len; i++)
+    {
+      GParameter *param = &g_array_index (construct_params, GParameter, i);
+
+      g_free ((gchar *) param->name);
+      g_value_unset (&param->value);
+    }
+
+  g_array_free (construct_params, TRUE);
+  g_list_free (members);
+
+  /* we use g_list_prepend() above, but we want to maintain
+   * the ordering of json_object_get_members() here
+   */
+  members = g_list_reverse (members_left);
+
+  /* do the Serializable type check once */
+  if (g_type_is_a (gtype, JSON_TYPE_SERIALIZABLE))
+    {
+      serializable = JSON_SERIALIZABLE (retval);
+      iface = JSON_SERIALIZABLE_GET_IFACE (serializable);
+      deserialize_property = (iface->deserialize_property != NULL);
+    }
+  else
+    deserialize_property = FALSE;
+
+  g_object_freeze_notify (retval);
+
+  for (l = members; l != NULL; l = l->next)
+    {
+      const gchar *member_name = l->data;
+      GParamSpec *pspec;
+      JsonNode *val;
+      GValue value = { 0, };
+      gboolean res = FALSE;
+
+      pspec = g_object_class_find_property (klass, member_name);
+      if (!pspec)
+        continue;
+
+      /* we should have dealt with these above */
+      if ((pspec->flags & G_PARAM_CONSTRUCT_ONLY) ||
+          (pspec->flags & G_PARAM_CONSTRUCT))
+        continue;
+
+      if (!(pspec->flags & G_PARAM_WRITABLE))
+        continue;
+
+      g_value_init (&value, G_PARAM_SPEC_VALUE_TYPE (pspec));
+
+      val = json_object_get_member (object, member_name);
+
+      if (deserialize_property)
+        {
+          res = iface->deserialize_property (serializable, pspec->name,
+                                             &value,
+                                             pspec,
+                                             val);
+        }
+
+      if (!res)
+        res = json_deserialize_pspec (&value, pspec, val);
+
+      /* FIXME - we probably want to be able to have a custom
+       * set_property() for Serializable implementations
+       */
+      if (res)
+        g_object_set_property (retval, pspec->name, &value);
+
+      g_value_unset (&value);
+    }
+
+  g_list_free (members);
+
+  g_object_thaw_notify (retval);
+
+  g_type_class_unref (klass);
+
+  return retval;
+}
+
+static JsonObject *
+json_gobject_dump (GObject *gobject)
+{
+  JsonSerializableIface *iface = NULL;
+  JsonSerializable *serializable = NULL;
+  gboolean serialize_property = FALSE;
+  JsonObject *object;
+  GParamSpec **pspecs;
+  guint n_pspecs, i;
+
+  if (JSON_IS_SERIALIZABLE (gobject))
+    {
+      serializable = JSON_SERIALIZABLE (gobject);
+      iface = JSON_SERIALIZABLE_GET_IFACE (gobject);
+      serialize_property = (iface->serialize_property != NULL);
+    }
+
+  object = json_object_new ();
+
+  pspecs = g_object_class_list_properties (G_OBJECT_GET_CLASS (gobject), &n_pspecs);
+  for (i = 0; i < n_pspecs; i++)
+    {
+      GParamSpec *pspec = pspecs[i];
+      GValue value = { 0, };
+      JsonNode *node = NULL;
+
+      /* read only what we can */
+      if (!(pspec->flags & G_PARAM_READABLE))
+        continue;
+
+      g_value_init (&value, G_PARAM_SPEC_VALUE_TYPE (pspec));
+      g_object_get_property (gobject, pspec->name, &value);
+
+      /* if there is a serialization vfunc, then it is completely responsible
+       * for serializing the property, possibly by calling the implementation
+       * of the default JsonSerializable interface through chaining up
+       */
+      if (serialize_property)
+        {
+          node = iface->serialize_property (serializable, pspec->name,
+                                            &value,
+                                            pspec);
+        }
+      else
+        node = json_serialize_pspec (&value, pspec);
+
+      if (node)
+        json_object_set_member (object, pspec->name, node);
+
+      g_value_unset (&value);
+    }
+
+  g_free (pspecs);
+
+  return object;
+}
+
+gboolean
 json_deserialize_pspec (GValue     *value,
                         GParamSpec *pspec,
                         JsonNode   *node)
@@ -181,10 +392,37 @@ json_deserialize_pspec (GValue     *value,
   GValue node_value = { 0, };
   gboolean retval = FALSE;
 
+  if (G_TYPE_FUNDAMENTAL (G_VALUE_TYPE (value)) == G_TYPE_BOXED)
+    {
+      JsonNodeType node_type = json_node_get_node_type (node);
+      GType boxed_type = G_VALUE_TYPE (value);
+
+      if (json_boxed_can_deserialize (boxed_type, node_type))
+        {
+          gpointer boxed = json_boxed_deserialize (boxed_type, node);
+
+          g_value_take_boxed (value, boxed);
+
+          return TRUE;
+        }
+    }
+
   switch (JSON_NODE_TYPE (node))
     {
     case JSON_NODE_OBJECT:
-      return FALSE;
+      if (g_type_is_a (G_VALUE_TYPE (value), G_TYPE_OBJECT))
+        {
+          GObject *object;
+
+          object = json_gobject_new (G_VALUE_TYPE (value), json_node_get_object (node));
+          if (object != NULL)
+            g_value_take_object (value, object);
+          else
+            g_value_set_object (value, NULL);
+
+          retval = TRUE;
+        }
+      break;
 
     case JSON_NODE_ARRAY:
       if (G_VALUE_HOLDS (value, G_TYPE_STRV))
@@ -200,7 +438,7 @@ json_deserialize_pspec (GValue     *value,
               if (JSON_NODE_TYPE (val) != JSON_NODE_VALUE)
                 continue;
 
-              if (json_node_get_string (val) != NULL);
+              if (json_node_get_string (val) != NULL)
                 g_ptr_array_add (str_array, (gpointer) json_node_get_string (val));
             }
 
@@ -321,12 +559,13 @@ json_deserialize_pspec (GValue     *value,
   return retval;
 }
 
-static JsonNode *
+JsonNode *
 json_serialize_pspec (const GValue *real_value,
                       GParamSpec   *pspec)
 {
   JsonNode *retval = NULL;
   GValue value = { 0, };
+  JsonNodeType node_type;
 
   switch (G_TYPE_FUNDAMENTAL (G_VALUE_TYPE (real_value)))
     {
@@ -341,18 +580,8 @@ json_serialize_pspec (const GValue *real_value,
       g_value_unset (&value);
       break;
 
-    case G_TYPE_INT:
-      retval = json_node_new (JSON_NODE_VALUE);
-      json_node_set_int (retval, g_value_get_int (real_value));
-      break;
-
-    case G_TYPE_FLOAT:
-      retval = json_node_new (JSON_NODE_VALUE);
-      json_node_set_double (retval, g_value_get_float (real_value));
-      break;
-
     case G_TYPE_STRING:
-      /* strings might be NULL */
+      /* strings might be NULL, so we handle it differently */
       if (!g_value_get_string (real_value))
         retval = json_node_new (JSON_NODE_NULL);
       else
@@ -363,6 +592,16 @@ json_serialize_pspec (const GValue *real_value,
         }
       break;
 
+    case G_TYPE_INT:
+      retval = json_node_new (JSON_NODE_VALUE);
+      json_node_set_int (retval, g_value_get_int (real_value));
+      break;
+
+    case G_TYPE_FLOAT:
+      retval = json_node_new (JSON_NODE_VALUE);
+      json_node_set_double (retval, g_value_get_float (real_value));
+      break;
+
     case G_TYPE_BOXED:
       if (G_VALUE_HOLDS (real_value, G_TYPE_STRV))
         {
@@ -384,11 +623,15 @@ json_serialize_pspec (const GValue *real_value,
           retval = json_node_new (JSON_NODE_ARRAY);
           json_node_take_array (retval, array);
         }
-      else
+      else if (json_boxed_can_serialize (G_VALUE_TYPE (real_value), &node_type))
         {
-          g_warning ("Unsupported type `%s'",
-                     g_type_name (G_VALUE_TYPE (real_value)));
+          gpointer boxed = g_value_get_boxed (real_value);
+
+          retval = json_boxed_serialize (G_VALUE_TYPE (real_value), boxed);
         }
+      else
+        g_warning ("Boxed type '%s' is not handled by JSON-GLib",
+                   g_type_name (G_VALUE_TYPE (real_value)));
       break;
 
     case G_TYPE_UINT:
@@ -426,13 +669,26 @@ json_serialize_pspec (const GValue *real_value,
       json_node_set_int (retval, g_value_get_flags (real_value));
       break;
 
+    case G_TYPE_OBJECT:
+      {
+        GObject *object = g_value_get_object (real_value);
+
+        if (object != NULL)
+          {
+            retval = json_node_new (JSON_NODE_OBJECT);
+            json_node_take_object (retval, json_gobject_dump (object));
+          }
+        else
+          retval = json_node_new (JSON_NODE_NULL);
+      }
+      break;
+
     case G_TYPE_NONE:
       retval = json_node_new (JSON_NODE_NULL);
       break;
 
     default:
-      g_warning ("Unsupported type `%s'",
-                 g_type_name (G_VALUE_TYPE (real_value)));
+      g_warning ("Unsupported type `%s'", g_type_name (G_VALUE_TYPE (real_value)));
       break;
     }
 
@@ -440,128 +696,91 @@ json_serialize_pspec (const GValue *real_value,
 }
 
 /**
- * json_serializable_serialize_property:
- * @serializable: a #JsonSerializable object
- * @property_name: the name of the property
- * @value: the value of the property
- * @pspec: a #GParamSpec
+ * json_gobject_deserialize:
+ * @gtype: the type of the #GObject to create
+ * @node: a #JsonNode of type %JSON_NODE_OBJECT describing the
+ *   instance of type @gtype
  *
- * Asks a #JsonSerializable implementation to serialize a #GObject
- * property into a #JsonNode object.
+ * Creates a new #GObject of type @gtype, and constructs it
+ * using the members of the passed #JsonObject
  *
- * Return value: a #JsonNode containing the serialize property
+ * Return value: (transfer full): The newly created #GObject
+ *   instance. Use g_object_unref() to free the resources
+ *   allocated by this function
+ *
+ * Since: 0.10
  */
-JsonNode *
-json_serializable_serialize_property (JsonSerializable *serializable,
-                                      const gchar      *property_name,
-                                      const GValue     *value,
-                                      GParamSpec       *pspec)
+GObject *
+json_gobject_deserialize (GType     gtype,
+                          JsonNode *node)
 {
-  JsonSerializableIface *iface;
+  g_return_val_if_fail (g_type_is_a (gtype, G_TYPE_OBJECT), NULL);
+  g_return_val_if_fail (JSON_NODE_TYPE (node) == JSON_NODE_OBJECT, NULL);
 
-  g_return_val_if_fail (JSON_IS_SERIALIZABLE (serializable), NULL);
-  g_return_val_if_fail (property_name != NULL, NULL);
-  g_return_val_if_fail (value != NULL, NULL);
-  g_return_val_if_fail (pspec != NULL, NULL);
-
-  iface = JSON_SERIALIZABLE_GET_IFACE (serializable);
-
-  return iface->serialize_property (serializable, property_name, value, pspec);
+  return json_gobject_new (gtype, json_node_get_object (node));
 }
 
 /**
- * json_serializable_deserialize_property:
- * @serializable: a #JsonSerializable
- * @property_name: the name of the property
- * @value: a pointer to an uninitialized #GValue
- * @pspec: a #GParamSpec
- * @property_node: a #JsonNode containing the serialized property
+ * json_gobject_serialize:
+ * @gobject: a #GObject
  *
- * Asks a #JsonSerializable implementation to deserialize the
- * property contained inside @property_node into @value.
+ * Creates a #JsonNode representing the passed #GObject
+ * instance. Each member of the returned JSON object will
+ * map to a property of the #GObject
  *
- * Return value: %TRUE if the property was successfully deserialized.
+ * Return value: (transfer full): the newly created #JsonNode
+ *   of type %JSON_NODE_OBJECT. Use json_node_free() to free
+ *   the resources allocated by this function
+ *
+ * Since: 0.10
  */
-gboolean
-json_serializable_deserialize_property (JsonSerializable *serializable,
-                                        const gchar      *property_name,
-                                        GValue           *value,
-                                        GParamSpec       *pspec,
-                                        JsonNode         *property_node)
+JsonNode *
+json_gobject_serialize (GObject *gobject)
 {
-  JsonSerializableIface *iface;
+  JsonNode *retval;
 
-  g_return_val_if_fail (JSON_IS_SERIALIZABLE (serializable), FALSE);
-  g_return_val_if_fail (property_name != NULL, FALSE);
-  g_return_val_if_fail (value != NULL, FALSE);
-  g_return_val_if_fail (pspec != NULL, FALSE);
-  g_return_val_if_fail (property_node != NULL, FALSE);
+  g_return_val_if_fail (G_IS_OBJECT (gobject), NULL);
 
-  iface = JSON_SERIALIZABLE_GET_IFACE (serializable);
+  retval = json_node_new (JSON_NODE_OBJECT);
+  json_node_take_object (retval, json_gobject_dump (gobject));
 
-  return iface->deserialize_property (serializable,
-                                      property_name,
-                                      value,
-                                      pspec,
-                                      property_node);
-}
-
-static gboolean
-json_serializable_real_deserialize (JsonSerializable *serializable,
-                                    const gchar      *name,
-                                    GValue           *value,
-                                    GParamSpec       *pspec,
-                                    JsonNode         *node)
-{
-  return json_deserialize_pspec (value, pspec, node);
-}
-
-static JsonNode *
-json_serializable_real_serialize (JsonSerializable *serializable,
-                                  const gchar      *name,
-                                  const GValue     *value,
-                                  GParamSpec       *pspec)
-{
-  return json_serialize_pspec (value, pspec);
-}
-
-static void
-json_serializable_base_init (gpointer g_class,
-                             gpointer data)
-{
-  static gboolean is_initialized = FALSE;
-
-  if (G_UNLIKELY (!is_initialized))
-    {
-      JsonSerializableIface *iface = g_class;
-
-      iface->serialize_property = json_serializable_real_serialize;
-      iface->deserialize_property = json_serializable_real_deserialize;
-
-      is_initialized = TRUE;
-    }
-}
-
-GType
-json_serializable_get_type (void)
-{
-  static GType iface_type = 0;
-
-  if (!iface_type)
-    iface_type =
-      g_type_register_static_simple (G_TYPE_INTERFACE,
-                                     g_intern_static_string ("JsonSerializable"),
-                                     sizeof (JsonSerializableIface),
-                                     json_serializable_base_init,
-                                     0, NULL, 0);
-
-  return iface_type;
+  return retval;
 }
 
 /**
  * json_construct_gobject:
  * @gtype: the #GType of object to construct
  * @data: a JSON data stream
+ * @length: length of the data stream
+ * @error: return location for a #GError, or %NULL
+ *
+ * Deserializes a JSON data stream and creates the corresponding
+ * #GObject class. If @gtype implements the #JsonSerializableIface
+ * interface, it will be asked to deserialize all the JSON members
+ * into the respective properties; otherwise, the default implementation
+ * will be used to translate the compatible JSON native types.
+ *
+ * Note: the JSON data stream must be an object declaration.
+ *
+ * Return value: a #GObject or %NULL
+ *
+ * Since: 0.4
+ *
+ * Deprecated: 0.10: Use json_gobject_from_data() instead
+ */
+GObject *
+json_construct_gobject (GType         gtype,
+                        const gchar  *data,
+                        gsize         length,
+                        GError      **error)
+{
+  return json_gobject_from_data (gtype, data, strlen (data), error);
+}
+
+/**
+ * json_gobject_from_data:
+ * @gtype: the #GType of object to construct
+ * @data: a JSON data stream
  * @length: length of the data stream, or -1 if it is NUL-terminated
  * @error: return location for a #GError, or %NULL
  *
@@ -575,24 +794,17 @@ json_serializable_get_type (void)
  *
  * Return value: a #GObject or %NULL
  *
- * Since: 0.4
+ * Since: 0.10
  */
 GObject *
-json_construct_gobject (GType         gtype,
+json_gobject_from_data (GType         gtype,
                         const gchar  *data,
-                        gsize         length,
+                        gssize        length,
                         GError      **error)
 {
-  JsonSerializableIface *iface = NULL;
-  JsonSerializable *serializable = NULL;
-  gboolean deserialize_property;
   JsonParser *parser;
   JsonNode *root;
-  JsonObject *object;
   GError *parse_error;
-  GList *members, *l;
-  guint n_members;
-  GObjectClass *klass;
   GObject *retval;
 
   g_return_val_if_fail (gtype != G_TYPE_INVALID, NULL);
@@ -624,69 +836,8 @@ json_construct_gobject (GType         gtype,
       return NULL;
     }
 
-  klass = g_type_class_ref (gtype);
-  retval = g_object_new (gtype, NULL);
+  retval = json_gobject_deserialize (gtype, root);
 
-  if (g_type_is_a (gtype, JSON_TYPE_SERIALIZABLE))
-    {
-      serializable = JSON_SERIALIZABLE (retval);
-      iface = JSON_SERIALIZABLE_GET_IFACE (serializable);
-      deserialize_property = (iface->deserialize_property != NULL);
-    }
-  else
-    deserialize_property = FALSE;
-
-  object = json_node_get_object (root);
-
-  g_object_freeze_notify (retval);
-
-  n_members = json_object_get_size (object);
-  members = json_object_get_members (object);
-
-  for (l = members; l != NULL; l = l->next)
-    {
-      const gchar *member_name = l->data;
-      GParamSpec *pspec;
-      JsonNode *val;
-      GValue value = { 0, };
-      gboolean res = FALSE;
-
-      pspec = g_object_class_find_property (klass, member_name);
-      if (!pspec)
-        continue;
-
-      if (pspec->flags & G_PARAM_CONSTRUCT_ONLY)
-        continue;
-
-      if (!(pspec->flags & G_PARAM_WRITABLE))
-        continue;
-
-      g_value_init (&value, G_PARAM_SPEC_VALUE_TYPE (pspec));
-
-      val = json_object_get_member (object, member_name);
-
-      if (deserialize_property)
-        {
-          res = iface->deserialize_property (serializable, pspec->name,
-                                             &value,
-                                             pspec,
-                                             val);
-        }
-
-      if (!res)
-        res = json_deserialize_pspec (&value, pspec, val);
-
-      if (res)
-        g_object_set_property (retval, pspec->name, &value);
-
-      g_value_unset (&value);
-    }
-
-  g_list_free (members);
-
-  g_object_thaw_notify (retval);
-
-  g_type_class_unref (klass);
   g_object_unref (parser);
 
   return retval;
@@ -695,7 +846,7 @@ json_construct_gobject (GType         gtype,
 /**
  * json_serialize_gobject:
  * @gobject: a #GObject
- * @length: return value for the length of the buffer, or %NULL
+ * @length: (out): return value for the length of the buffer, or %NULL
  *
  * Serializes a #GObject into a JSON data stream. If @gobject implements
  * the #JsonSerializableIface interface, it will be asked to serizalize all
@@ -703,67 +854,44 @@ json_construct_gobject (GType         gtype,
  * translate the compatible types into JSON native types.
  *
  * Return value: a JSON data stream representing the passed #GObject
+ *
+ * Deprecated: 0.10: Use json_gobject_to_data() instead
  */
 gchar *
 json_serialize_gobject (GObject *gobject,
                         gsize   *length)
 {
-  JsonSerializableIface *iface = NULL;
-  JsonSerializable *serializable = NULL;
-  gboolean serialize_property = FALSE;
+  return json_gobject_to_data (gobject, length);
+}
+
+/**
+ * json_gobject_to_data:
+ * @gobject: a #GObject
+ * @length: (out): return value for the length of the buffer, or %NULL
+ *
+ * Serializes a #GObject into a JSON data stream, iterating recursively
+ * over each property.
+ *
+ * If @gobject implements the #JsonSerializableIface interface, it will
+ * be asked to serialize all its properties; otherwise, the default
+ * implementation will be use to translate the compatible types into
+ * JSON native types.
+ *
+ * Return value: a JSON data stream representing the passed #GObject
+ *
+ * Since: 0.10
+ */
+gchar *
+json_gobject_to_data (GObject *gobject,
+                      gsize   *length)
+{
   JsonGenerator *gen;
   JsonNode *root;
-  JsonObject *object;
-  GParamSpec **pspecs;
-  guint n_pspecs, i;
   gchar *data;
 
   g_return_val_if_fail (G_OBJECT (gobject), NULL);
 
-  if (JSON_IS_SERIALIZABLE (gobject))
-    {
-      serializable = JSON_SERIALIZABLE (gobject);
-      iface = JSON_SERIALIZABLE_GET_IFACE (gobject);
-      serialize_property = (iface->serialize_property != NULL);
-    }
-
-  object = json_object_new ();
-
-  root = json_node_new (JSON_NODE_OBJECT);
-  json_node_take_object (root, object);
-
-  pspecs = g_object_class_list_properties (G_OBJECT_GET_CLASS (gobject),
-                                           &n_pspecs);
-  for (i = 0; i < n_pspecs; i++)
-    {
-      GParamSpec *pspec = pspecs[i];
-      GValue value = { 0, };
-      JsonNode *node = NULL;
-
-      /* read only what we can */
-      if (!(pspec->flags & G_PARAM_READABLE))
-        continue;
-
-      g_value_init (&value, G_PARAM_SPEC_VALUE_TYPE (pspec));
-      g_object_get_property (gobject, pspec->name, &value);
-
-      if (serialize_property)
-        {
-          node = iface->serialize_property (serializable, pspec->name,
-                                            &value,
-                                            pspec);
-        }
-
-      if (!node)
-        node = json_serialize_pspec (&value, pspec);
-
-      if (node)
-        json_object_set_member (object, pspec->name, node);
-
-      g_value_unset (&value);
-    }
-
-  g_free (pspecs);
+  root = json_gobject_serialize (gobject);
 
   gen = g_object_new (JSON_TYPE_GENERATOR,
                       "root", root,
@@ -773,6 +901,7 @@ json_serialize_gobject (GObject *gobject,
 
   data = json_generator_to_data (gen, length);
   g_object_unref (gen);
+
   json_node_free (root);
 
   return data;
